local addon, w = ...
local buttons = WonderBarButtons
local icons = WonderBarIcons
local actions = WonderBarActions
local handler = WonderBarHandler
local LBF

if ( LibStub ) then
	LBF = LibStub("LibButtonFacade", true)
end





local function UpdateRangeIndicator(button, action)
	local b = IsActionInRange(action, "target")
	if ( not b or b == 1 ) then
		button.range:Hide()
	else
		button.range:Show()
	end

end

function ShowRangeIndicator()
	for action, button in next, actions do
		UpdateRangeIndicator(button, action)
	end
	return 0
end

function HideRangeIndicator()
	for action, button in next, actions do
		button.range:Hide()
	end
end

function RangeIndicatorOnEvent()
	if ( UnitExists("target") ) then
		TimerInit("range", ShowRangeIndicator, 0) 
	else
		TimerInit("range", HideRangeIndicator, 0) 
	end
end

local function UpdateCooldown(button, action)
	local start, duration, enable = GetActionCooldown(action)
	if ( start > 0 and duration > 1.5 and enable > 0 ) then
		button.cooldown:SetCooldown(start, duration)
		button.cooldown:Show()
	else
		button.cooldown:Hide()
	end
end

local function UpdateUsable(button, action)
	local isUsable, notEnoughMana = IsUsableAction(action)
	if ( isUsable ) then
		button:SetAlpha(1)
	elseif ( notEnoughMana ) then
		button:SetAlpha(0.5)
	else
		button:SetAlpha(0.5)
	end
	if ( IsConsumableAction(action) or IsStackableAction(action) ) then
		button.count:SetText(GetActionCount(action))
		button.count:Show()
	else
		button.count:Hide()
	end

end

local function UpdateState(button, action)
	if ( IsCurrentAction(action) or IsAutoRepeatAction(action) ) then
		button.state:Show()
	else
		button.state:Hide()
	end
end


local function UpdateEquiped(button, action)	
	if ( IsEquippedAction(action) ) then
		button.equiped:Show()
	else
		button.equiped:Hide()
	end
end





local function UpdateButton(button, action)
	if ( HasAction(action) ) then
		actions[action] = button
		UpdateState(button, action)
		UpdateUsable(button, action)
		UpdateCooldown(button, action)
		UpdateEquiped(button, action)
		if ( WonderBarOptions.range ) then
			UpdateRangeIndicator(button, action)
		end

		if ( GetActionInfo(action) == "flyout" ) then
			ActionButton_UpdateFlyout(button)
		else
			button.FlyoutArrow:Hide()
			button.FlyoutBorder:Hide()
		end
		ActionButton_UpdateOverlayGlow(button)
	else
		actions[action] = nil
		ActionButton_HideOverlayGlow(button)
		button:SetAlpha(0.25)
		button.cooldown:Hide()
		button.state:Hide()
		button.range:Hide()
		button.count:Hide()
		button.equiped:Hide()
		button.FlyoutArrow:Hide()
	end
end

local function UpdateAction(_, _, action)
	local button = buttons[action]
	if ( not button ) then 
		return
	else
		local icon = icons[action]
		icon:SetTexture(GetActionTexture(action))
		if ( icon:IsShown() ) then
			UpdateButton(button, action)
		end
	end
end




RegisterEvent(function(_, _, arg1)
	for k, v in next, actions do
		local actionType, id, subType = GetActionInfo(v.action);
		if ( actionType == "spell" and id == arg1 ) then
			ActionButton_UpdateOverlayGlow(v)
		end
	end
end, "SPELL_ACTIVATION_OVERLAY_GLOW_SHOW")

RegisterEvent(function(_, _, arg1)
	for k, v in next, actions do
		local actionType, id, subType = GetActionInfo(v.action);
		if ( actionType == "spell" and id == arg1 ) then
			ActionButton_HideOverlayGlow(v)
		end
	end
end, "SPELL_ACTIVATION_OVERLAY_GLOW_HIDE")

RegisterEvent(UpdateAction, "ACTIONBAR_SLOT_CHANGED")


local f = function()
	for k, v in next, buttons do
		UpdateAction(_, _, k)
	end
end

RegisterEvent(function()
	TimerInit(f, f, 1)

end, "PLAYER_ENTERING_WORLD", "PLAYER_TALENT_UPDATE")



RegisterEvent(function(self, event, arg1)

	for k, v in next, actions do
		UpdateState(v, k)
	end
end, "ACTIONBAR_UPDATE_STATE", "STOP_AUTOREPEAT_SPELL", "START_AUTOREPEAT_SPELL")

RegisterEvent(function()
	for k, v in next, actions do
		if ( IsAttackAction(k) ) then
			v.state:Show()
		end
	end
end, "PLAYER_ENTER_COMBAT")

RegisterEvent(function()
	for k, v in next, actions do
		if ( IsAttackAction(k) ) then 
			v.state:Hide()
		end
	end
end, "PLAYER_LEAVE_COMBAT")

RegisterEvent(function(_, _, ...)
	for k, v in next, actions do
		UpdateCooldown(v, k)
	end
end, "ACTIONBAR_UPDATE_COOLDOWN")

RegisterEvent(function()
	for k, v in next, actions do
		UpdateUsable(v, k)
	end
end, "ACTIONBAR_UPDATE_USABLE")


local function ShowGameTooltip(self)
	local parent = self:GetParent()
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	if ( parent:GetName() ~= addon.."PetBar" ) then
		local action = self:GetAttribute("action")
		if ( HasAction(action) ) then
			GameTooltip:SetAction(action)
		else
			GameTooltip:SetText("Action "..string.color(action, "ffff00"), 1.0, 1.0, 1.0)
		end
	elseif ( self.tooltip ) then
		GameTooltip:SetText(self.tooltip, 1.0, 1.0, 1.0)
	else
		GameTooltip:SetPetAction(self:GetAttribute("*action1"))
	end
	GameTooltip:AddLine("\n")
	local b = self:GetName()
	local binds = ""
	for k, v in next, WonderBarBinds do
		if ( b == v ) then
			binds = binds.." "..k..","
		end
	end	
	if ( binds ~= "" ) then
		GameTooltip:AddLine("Bind :"..binds:sub(1, #binds-1):color("ffff00").."\n", 1.0, 1.0, 1.0)
	end
	GameTooltip:AddLine(string.color("CTRL + Button Left" ,"788221").." to move your bar|n", 1.0, 1.0, 1.0)
	GameTooltip:AddLine(string.color("CTRL + Button Right" ,"788221").." to bind/unbind a key|n", 1.0, 1.0, 1.0)
	GameTooltip:Show()
end


local DragMe = [[
	if ( not IsControlKeyDown() ) and ( not PlayerInCombat() or not lock ) then
		local type = self:GetAttribute("type") or self:GetAttribute("*type1")
		local action
		if ( type == "pet" ) then 
			type = "petaction" 
			action = self:GetAttribute("*action1")	
		else 
			action = self:GetAttribute("action")	
		end
		return type, action
	end
]]


local ToggleMe = [[
	if ( toggle == "Show" ) then
		toggle = "Hide"
	else
		toggle = "Show"
	end
	for i = 9, 11 do
	local b = buttons[i]
		b[toggle](b, true)
	end
]]




function w.NewButton(frame, i)
	local button = CreateFrame("Checkbutton", frame:GetName().."Button"..i, frame, "SecureActionButtonTemplate")

	button.cooldown = CreateFrame("Cooldown", button:GetName().."Cooldown", button, "CooldownFrameTemplate")


	button:SetScript("OnLeave", HideGameTooltip)
	button:SetScript("OnEnter", ShowGameTooltip)
	button:RegisterForClicks("AnyDown")

	

	button.UpdateTooltip = ShowGameTooltip
	button:SetAttribute("ctrl-type2", "kb")
	button.kb = w.EnableKeyBinder


	button:SetScript("OnMouseWheel", w.BindKey)
	button:SetScript("OnKeyDown", w.BindKey)
	button:EnableMouseWheel(false)
	button:EnableKeyboard(false)



	RegisterForAnchor(button, frame, true)



	button:SetPushedTexture("Interface\\AddOns\\"..addon.."\\Skin\\Pushed")
	button:SetHighlightTexture("Interface\\AddOns\\"..addon.."\\Skin\\Highlight")
	button:SetNormalTexture("Interface\\AddOns\\"..addon.."\\Skin\\Normal")


--[[
	  = button:CreateTexture(nil, "OVERLAY")
	state:SetAllPoints(button)
	state:SetTexture("Interface\\AddOns\\"..addon.."\\Skin\\Pushed")
	state:Hide()
	button.state = state


]]
	button:SetCheckedTexture("Interface\\AddOns\\"..addon.."\\Skin\\Pushed")

	local state = button:GetCheckedTexture()
	button.state = state


	-- support for button facade
	if ( LBF ) then
		LBF:Group(addon, frame:GetName()):AddButton(button)
	end

	if ( frame:GetName() ~= addon.."PetBar" ) then



		local equiped  = button:CreateTexture(button:GetName().."Border", "OVERLAY")
		equiped:SetAllPoints(button)
		equiped:SetTexture("Interface\\AddOns\\"..addon.."\\Skin\\Equiped")
		equiped:Hide()
		button.equiped = equiped
	


	
		button:SetAttribute("useparent-unit", true)




		local fs = button:CreateFontString(nil, "OVERLAY")
		fs:SetFontObject("NumberFontNormal")
		fs:SetPoint("BOTTOMRIGHT", -3, 4)	
		button.count = fs

		local range = button:CreateTexture(nil, "OVERLAY")
		range:SetBlendMode("BLEND")
		range:SetAllPoints(button)
		range:SetTexture("Interface\\AddOns\\"..addon.."\\Skin\\Range")
		range:Hide()
		button.range = range


		button.FlyoutArrow =  button:CreateTexture(nil, "OVERLAY", "ActionBarFlyoutButton-ArrowUp")
		button.FlyoutBorder = state
		button.FlyoutBorderShadow = state



		handler:WrapScript(button, "OnReceiveDrag", DragMe)
		handler:WrapScript(button, "OnDragStart", DragMe)
		handler:WrapScript(button, "PostClick", "control:Run(modifier, button)")   
		handler:WrapScript(button, "PreClick", "control:Run(unit, button)")   

		handler:setv(button, i, 'buttons', frame:GetName():lower(), 'bars')
		button:SetAttribute("type", "action")

	else
		local icon = button:CreateTexture(nil, "BACKGROUND")
		icon:SetPoint("CENTER")
		icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
		button.icon = icon



		local shine = CreateFrame("Frame", button:GetName().."Shine", button,"AutoCastShineTemplate")
		shine:SetPoint("CENTER")

		if ( i == 8 ) then
			handler:WrapScript(button, "OnClick", ToggleMe)
		else
			if ( i > 8 ) then
				button:Hide()
				handler:setv(button, i, 'buttons')
				i = i-1
			else
				handler:WrapScript(button, "OnDragStart", DragMe)
			end
			button:SetAttribute("*type1", "pet")
		end
		button:SetAttribute("*action1", i)
	end
	return button
end

w.UpdateButton = UpdateButton
